@@ -35,6 +35,8 @@
 int output_level = 0;
 int log_fd = 0;
 
+int is_lsslot_cmd = 1;
+
 extern int lsslot_chrp_cpu(void);
 
 /**
@@ -798,10 +800,9 @@ int lsslot_chrp_mem(void)
 	int lmb_offset = strlen(OFDT_BASE);
 
 	lmb_list = get_lmbs(LMB_NORMAL_SORT);
-	if (lmb_list == NULL || lmb_list->lmbs == NULL) {
-		free_lmbs(lmb_list);
+	if (lmb_list == NULL || lmb_list->lmbs == NULL)
 		return -1;
-	}
+
 	
 	if (lmb_list->drconf_buf) {
 		print_drconf_mem(lmb_list);
