@@ -24,7 +24,6 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#define _GNU_SOURCE
 #include <fcntl.h>
 #include <getopt.h>
 #include <stdlib.h>
@@ -740,12 +739,6 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
-	if (get_platform() != PLATFORM_PSERIES_LPAR) {
-		fprintf(stderr, "%s: is not supported on the %s platform\n",
-							argv[0], platform_name);
-		exit(1);
-	}
-
 	/* Make sure the error injection facility is available */
 	fd = open(EI_IBM_ERRINJCT, O_RDONLY);
 	if (fd == -1) {
