@@ -16,7 +16,6 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-#define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -35,6 +34,7 @@
 #include <sys/ptrace.h>
 #include <sys/wait.h>
 #include <sys/resource.h>
+#include <sys/param.h>
 
 #ifdef WITH_LIBRTAS
 #include <librtas.h>
@@ -46,15 +46,12 @@
 #endif
 
 #include <errno.h>
+#include "cpu_info_helpers.h"
 
 #define PPC64_CPU_VERSION	"1.2"
 
-#define SYSFS_CPUDIR	"/sys/devices/system/cpu/cpu%d"
-#define SYSFS_SUBCORES	"/sys/devices/system/cpu/subcores_per_core"
 #define DSCR_DEFAULT_PATH "/sys/devices/system/cpu/dscr_default"
-#define INTSERV_PATH	"/proc/device-tree/cpus/%s/ibm,ppc-interrupt-server#s"
 
-#define SYSFS_PATH_MAX		128
 #define MAX_NR_CPUS		1024
 #define DIAGNOSTICS_RUN_MODE	42
 #define CPU_OFFLINE		-1
@@ -64,7 +61,7 @@ struct cpu_freq {
 	int offline;
 	int counter;
 	pthread_t tid;
-	unsigned long long freq;
+	double freq;
 };
 
 #ifndef __NR_perf_event_open
@@ -79,54 +76,14 @@ static int threads_in_system = 0;
 
 static int do_info(void);
 
-static int test_sysattr(char *attribute, int perms)
-{
-	char path[SYSFS_PATH_MAX];
-	int i;
-
-	for (i = 0; i < threads_in_system; i++) {
-		sprintf(path, SYSFS_CPUDIR"/%s", i, attribute);
-		if (access(path, F_OK))
-			continue;
-
-		if (access(path, perms))
-			return 0;
-	}
-
-	return 1;
-}
-
 static int sysattr_is_readable(char *attribute)
 {
-	return test_sysattr(attribute, R_OK);
+	return __sysattr_is_readable(attribute, threads_in_system);
 }
 
 static int sysattr_is_writeable(char *attribute)
 {
-	return test_sysattr(attribute, W_OK);
-}
-
-static int get_attribute(char *path, const char *fmt, int *value)
-{
-	FILE *fp;
-	int rc;
-
-	rc = access(path, F_OK);
-	if (rc)
-		return -1;
-
-
-	fp = fopen(path, "r");
-	if (!fp)
-		return -1;
-
-	rc = fscanf(fp, fmt, value);
-	fclose(fp);
-
-	if (rc == EOF)
-		return -1;
-
-	return 0;
+	return __sysattr_is_writeable(attribute, threads_in_system);
 }
 
 static int set_attribute(const char *path, const char *fmt, int value)
@@ -155,19 +112,6 @@ close:
 	return rc;
 }
 
-static int cpu_online(int thread)
-{
-	char path[SYSFS_PATH_MAX];
-	int rc, online;
-
-	sprintf(path, SYSFS_CPUDIR"/online", thread);
-	rc = get_attribute(path, "%d", &online);
-	if (rc || !online)
-		return 0;
-
-	return 1;
-}
-
 static int get_system_attribute(char *attribute, const char *fmt, int *value,
 			 int *inconsistent)
 {
@@ -207,9 +151,9 @@ static int set_system_attribute(char *attribute, const char *fmt, int state)
 		sprintf(path, SYSFS_CPUDIR"/%s", i, attribute);
 		rc = set_attribute(path, fmt, state);
 		/* When a CPU is offline some sysfs files are removed from the CPU
-		 * directory, for example smt_snooze_delay and dscr. The absence of the
-		 * file is not an error, so detect and clear the error when
-		 * set_attribute indicates ENOENT. */
+		 * directory, for example dscr. The absence of the file is not
+		 * an error, so detect and clear the error when set_attribute
+		 * indicates ENOENT. */
 		if (rc == -1 && errno == ENOENT)
 			rc = errno = 0;
 		if (rc)
@@ -279,27 +223,6 @@ static int get_dscr(int *value, int *inconsistent)
 	return rc;
 }
 
-static int set_smt_snooze_delay(int delay)
-{
-	if (!sysattr_is_writeable("smt_snooze_delay")) {
-		perror("Cannot set smt snooze delay");
-		return -2;
-	}
-
-	return set_system_attribute("smt_snooze_delay", "%d", delay);
-}
-
-static int get_smt_snooze_delay(int *delay, int *inconsistent)
-{
-	if (!sysattr_is_readable("smt_snooze_delay")) {
-		perror("Cannot retrieve smt snooze delay");
-		return -2;
-	}
-
-	return get_system_attribute("smt_snooze_delay", "%d", delay,
-				    inconsistent);
-}
-
 static int online_thread(const char *path)
 {
 	return set_attribute(path, "%d", 1);
@@ -310,115 +233,35 @@ static int offline_thread(const char *path)
 	return set_attribute(path, "%d", 0);
 }
 
-static int is_subcore_capable(void)
-{
-	return access(SYSFS_SUBCORES, F_OK) == 0;
-}
-
-static int num_subcores(void)
-{
-	int rc, subcores;
-	rc = get_attribute(SYSFS_SUBCORES, "%d", &subcores);
-	if (rc)
-		return -1;
-	return subcores;
-}
-
-static int get_cpu_info(void)
-{
-	DIR *d;
-	struct dirent *de;
-	int first_cpu = 1;
-	int rc;
-	int subcores;
-
-	d = opendir("/proc/device-tree/cpus");
-	if (!d)
-		return -1;
-
-	while ((de = readdir(d)) != NULL) {
-		if (!strncmp(de->d_name, "PowerPC", 7)) {
-			if (first_cpu) {
-				struct stat sbuf;
-				char path[128];
-
-				sprintf(path, INTSERV_PATH, de->d_name);
-				rc = stat(path, &sbuf);
-				if (!rc)
-					threads_per_cpu = sbuf.st_size / 4;
-
-				first_cpu = 0;
-			}
-
-			cpus_in_system++;
-		}
-	}
-
-	closedir(d);
-	threads_in_system = cpus_in_system * threads_per_cpu;
-
-	subcores = num_subcores();
-	if (is_subcore_capable() && subcores > 0) {
-		threads_per_cpu /= subcores;
-		cpus_in_system *= subcores;
-	}
-	return 0;
-}
-
 static int is_smt_capable(void)
 {
-	struct stat sb;
-	char path[SYSFS_PATH_MAX];
-	int i;
-
-	for (i = 0; i < threads_in_system; i++) {
-		sprintf(path, SYSFS_CPUDIR"/smt_snooze_delay", i);
-		if (stat(path, &sb))
-			continue;
-		return 1;
-	}
-
-	return 0;
+	return __is_smt_capable(threads_per_cpu);
 }
 
-static int get_one_smt_state(int primary_thread)
+static int get_one_smt_state(int core)
 {
-	int thread_state;
-	int smt_state = 0;
-	int i;
-
-	for (i = 0; i < threads_per_cpu; i++) {
-		thread_state = cpu_online(primary_thread + i);
-		smt_state += thread_state;
-	}
-
-	return smt_state ? smt_state : -1;
+	return __get_one_smt_state(core, threads_per_cpu);
 }
 
 static int get_smt_state(void)
 {
-	int system_state = -1;
+	int smt_state = -1;
 	int i;
 
-	if (!sysattr_is_readable("online")) {
-		perror("Cannot retrieve smt state");
-		return -2;
-	}
-
-	for (i = 0; i < threads_in_system; i += threads_per_cpu) {
-		int cpu_state;
-
-		cpu_state = get_one_smt_state(i);
-		if (cpu_state == -1)
+	for (i = 0; i < cpus_in_system; i++) {
+		int cpu_state = get_one_smt_state(i);
+		if (cpu_state == 0)
 			continue;
 
-		if (system_state == -1)
-			system_state = cpu_state;
-		else if (system_state != cpu_state)
-			return -1;
+		if (smt_state == -1)
+			smt_state = cpu_state;
+		if (smt_state != cpu_state) {
+			smt_state = -1;
+			break;
+		}
 	}
 
-	return system_state;
+	return smt_state;
 }
 
 static int set_one_smt_state(int thread, int online_threads)
@@ -449,8 +292,6 @@ static int set_one_smt_state(int thread, int online_threads)
 static int set_smt_state(int smt_state)
 {
 	int i, j, rc;
-	int ssd, update_ssd = 1;
-	int inconsistent = 0;
 	int error = 0;
 
 	if (!sysattr_is_writeable("online")) {
@@ -458,10 +299,6 @@ static int set_smt_state(int smt_state)
 		return -1;
 	}
 
-	rc = get_smt_snooze_delay(&ssd, &inconsistent);
-	if (rc)
-		update_ssd = 0;
-
 	for (i = 0; i < threads_in_system; i += threads_per_cpu) {
 		/* Online means any thread on this core running, so check all
 		 * threads in the core, not just the first. */
@@ -479,9 +316,6 @@ static int set_smt_state(int smt_state)
 		}
 	}
 
-	if (update_ssd)
-		set_smt_snooze_delay(ssd);
-
 	if (error) {
 		fprintf(stderr, "One or more cpus could not be on/offlined\n");
 		return -1;
@@ -508,28 +342,21 @@ static int is_dscr_capable(void)
 	return 0;
 }
 
-static int do_smt(char *state)
+static int do_smt(char *state, bool numeric)
 {
 	int rc = 0;
-	int smt_state;
+	int smt_state = 0;
 
 	if (!is_smt_capable()) {
-		fprintf(stderr, "Machine is not SMT capable\n");
+		if (numeric)
+			printf("SMT=1\n");
+		else
+			fprintf(stderr, "Machine is not SMT capable\n");
 		return -1;
 	}
 
 	if (!state) {
-		smt_state = get_smt_state();
-
-		if (smt_state == -2)
-			return -1;
-
-		if (smt_state == 1)
-			printf("SMT is off\n");
-		else if (smt_state == -1)
-			printf("Inconsistent state: mix of ST and SMT cores\n");
-		else
-			printf("SMT=%d\n", smt_state);
+		rc = __do_smt(numeric, cpus_in_system, threads_per_cpu, true);
 	} else {
 		if (!strcmp(state, "on"))
 			smt_state = threads_per_cpu;
@@ -668,40 +495,6 @@ static int do_dscr(char *state, pid_t pid)
 	return rc;
 }
 
-static int do_smt_snooze_delay(char *state)
-{
-	int rc = 0;
-
-	if (!is_smt_capable()) {
-		fprintf(stderr, "Machine is not SMT capable\n");
-		return -1;
-	}
-
-	if (!state) {
-		int ssd, inconsistent = 0;
-		rc = get_smt_snooze_delay(&ssd, &inconsistent);
-		if (rc) {
-			if (inconsistent)
-				printf("Inconsistent smt_snooze_delay\n");
-			else
-				printf("Could not retrieve smt_snooze_delay\n");
-		} else {
-			printf("smt_snooze_delay is %d\n", ssd);
-		}
-	} else {
-		int delay;
-
-		if (!strcmp(state, "off"))
-			delay = -1;
-		else
-			delay = strtol(state, NULL, 0);
-
-		rc = set_smt_snooze_delay(delay);
-	}
-
-	return rc;
-}
-
 #ifdef WITH_LIBRTAS
 
 static int do_run_mode(char *run_mode)
@@ -779,7 +572,7 @@ static int do_run_mode(char *run_mode)
 
 #ifdef HAVE_LINUX_PERF_EVENT_H
 
-static int setup_counters(struct cpu_freq *cpu_freqs)
+static int setup_counters(struct cpu_freq *cpu_freqs, int max_thread)
 {
 	int i;
 	struct perf_event_attr attr;
@@ -790,7 +583,10 @@ static int setup_counters(struct cpu_freq *cpu_freqs)
 	attr.disabled = 1;
 	attr.size = sizeof(attr);
 
-	for (i = 0; i < threads_in_system; i++) {
+	/* Record how long the event ran for */
+	attr.read_format |= PERF_FORMAT_TOTAL_TIME_RUNNING;
+
+	for (i = 0; i < max_thread; i++) {
 		if (!cpu_online(i)) {
 			cpu_freqs[i].offline = 1;
 			continue;
@@ -813,11 +609,11 @@ static int setup_counters(struct cpu_freq *cpu_freqs)
 	return 0;
 }
 
-static void start_counters(struct cpu_freq *cpu_freqs)
+static void start_counters(struct cpu_freq *cpu_freqs, int max_thread)
 {
 	int i;
 
-	for (i = 0; i < threads_in_system; i++) {
+	for (i = 0; i < max_thread; i++) {
 		if (cpu_freqs[i].offline)
 			continue;
 
@@ -825,11 +621,11 @@ static void start_counters(struct cpu_freq *cpu_freqs)
 	}
 }
 
-static void stop_counters(struct cpu_freq *cpu_freqs)
+static void stop_counters(struct cpu_freq *cpu_freqs, int max_thread)
 {
 	int i;
 
-	for (i = 0; i < threads_in_system; i++) {
+	for (i = 0; i < max_thread; i++) {
 		if (cpu_freqs[i].offline)
 			continue;
 
@@ -837,29 +633,42 @@ static void stop_counters(struct cpu_freq *cpu_freqs)
 	}
 }
 
-static void read_counters(struct cpu_freq *cpu_freqs)
+struct read_format {
+	uint64_t value;
+	uint64_t time_running;
+};
+
+static void read_counters(struct cpu_freq *cpu_freqs, int max_thread)
 {
 	int i;
+	struct read_format vals;
 
-	for (i = 0; i < threads_in_system; i++) {
+	for (i = 0; i < max_thread; i++) {
 		size_t res;
 
 		if (cpu_freqs[i].offline)
 			continue;
 
-		res = read(cpu_freqs[i].counter, &cpu_freqs[i].freq,
-			   sizeof(unsigned long long));
-		assert(res == sizeof(unsigned long long));
+		res = read(cpu_freqs[i].counter, &vals, sizeof(vals));
+		assert(res == sizeof(vals));
+
+		/* Warn if we don't get at least 0.1s of time on the CPU */
+		if (vals.time_running < 100000000) {
+			fprintf(stderr, "Measurement interval was too small, is someone running perf?\n");
+			exit(1);
+		}
+
+		cpu_freqs[i].freq = 1.0 * vals.value / vals.time_running;
 
 		close(cpu_freqs[i].counter);
 	}
 }
 
-static void check_threads(struct cpu_freq *cpu_freqs)
+static void check_threads(struct cpu_freq *cpu_freqs, int max_thread)
 {
 	int i;
 
-	for (i = 0; i < threads_in_system; i++) {
+	for (i = 0; i < max_thread; i++) {
 		if (cpu_freqs[i].offline)
 			continue;
 
@@ -978,33 +787,37 @@ static void setrlimit_open_files(void)
 	setrlimit(RLIMIT_NOFILE, &new_rlim);
 }
 
-#define freq_calc(cycles, time)	(1.0 * (cycles) / (time) / 1000000000ULL)
-
 static int do_cpu_frequency(int sleep_time)
 {
 	int i, rc;
-	unsigned long long min = -1ULL;
+	double min = -1ULL;
 	unsigned long min_cpu = -1UL;
-	unsigned long long max = 0;
+	double max = 0;
 	unsigned long max_cpu = -1UL;
-	unsigned long long sum = 0;
+	double sum = 0;
 	unsigned long count = 0;
 	struct cpu_freq *cpu_freqs;
+	int max_thread;
 
 	setrlimit_open_files();
 
-	cpu_freqs = calloc(threads_in_system, sizeof(*cpu_freqs));
+	max_thread = MIN(threads_in_system, CPU_SETSIZE);
+	if (max_thread < threads_in_system)
+		printf("ppc64_cpu currently supports up to %d CPUs\n",
+			CPU_SETSIZE);
+
+	cpu_freqs = calloc(max_thread, sizeof(*cpu_freqs));
 	if (!cpu_freqs)
 		return -ENOMEM;
 
-	rc = setup_counters(cpu_freqs);
+	rc = setup_counters(cpu_freqs, max_thread);
 	if (rc) {
 		free(cpu_freqs);
 		return rc;
 	}
 
 	/* Start a soak thread on each CPU */
-	for (i = 0; i < threads_in_system; i++) {
+	for (i = 0; i < max_thread; i++) {
 		if (cpu_freqs[i].offline)
 			continue;
 
@@ -1019,16 +832,16 @@ static int do_cpu_frequency(int sleep_time)
 	/* Wait for soak threads to start */
 	usleep(1000000);
 
-	start_counters(cpu_freqs);
+	start_counters(cpu_freqs, max_thread);
 	/* Count for specified timeout in seconds */
 	usleep(sleep_time * 1000000);
 
-	stop_counters(cpu_freqs);
-	check_threads(cpu_freqs);
-	read_counters(cpu_freqs);
+	stop_counters(cpu_freqs, max_thread);
+	check_threads(cpu_freqs, max_thread);
+	read_counters(cpu_freqs, max_thread);
 
-	for (i = 0; i < threads_in_system; i++) {
-		unsigned long long frequency;
+	for (i = 0; i < max_thread; i++) {
+		double frequency;
 
 		if (cpu_freqs[i].offline)
 			continue;
@@ -1048,11 +861,9 @@ static int do_cpu_frequency(int sleep_time)
 	}
 
 	report_system_power_mode();
-	printf("min:\t%.3f GHz (cpu %ld)\n", freq_calc(min, sleep_time),
-	       min_cpu);
-	printf("max:\t%.3f GHz (cpu %ld)\n", freq_calc(max, sleep_time),
-	       max_cpu);
-	printf("avg:\t%.3f GHz\n\n", freq_calc((sum / count), sleep_time));
+	printf("min:\t%.3f GHz (cpu %ld)\n", min, min_cpu);
+	printf("max:\t%.3f GHz (cpu %ld)\n", max, max_cpu);
+	printf("avg:\t%.3f GHz\n\n", sum / count);
 
 	free(cpu_freqs);
 	return 0;
@@ -1060,7 +871,7 @@ static int do_cpu_frequency(int sleep_time)
 
 #else
 
-static int do_cpu_frequency(void)
+static int do_cpu_frequency(int sleep_time)
 {
 	printf("CPU Frequency determination is not supported on this "
 	       "platfom.\n");
@@ -1130,19 +941,13 @@ static int do_online_cores(char *cores, int state)
 	}
 
 	smt_state = get_smt_state();
-	if (smt_state == -1) {
-		printf("Bad or inconsistent SMT state: use ppc64_cpu --smt=on|off to set all\n"
-                       "cores to have the same number of online threads to continue.\n");
-		do_info();
-		return -1;
-	}
 
 	core_state = calloc(cpus_in_system, sizeof(int));
 	if (!core_state)
 		return -ENOMEM;
 
 	for (i = 0; i < cpus_in_system ; i++)
-		core_state[i] = cpu_online(i * threads_per_cpu);
+		core_state[i] = (get_one_smt_state(i) > 0);
 
 	if (!cores) {
 		printf("Cores %s = ", state == 0 ? "offline" : "online");
@@ -1160,6 +965,13 @@ static int do_online_cores(char *cores, int state)
 		return 0;
 	}
 
+	if (smt_state == -1) {
+		printf("Bad or inconsistent SMT state: use ppc64_cpu --smt=on|off to set all\n"
+                       "cores to have the same number of online threads to continue.\n");
+		do_info();
+		return -1;
+	}
+
 	desired_core_state = calloc(cpus_in_system, sizeof(int));
 	if (!desired_core_state) {
 		free(core_state);
@@ -1186,7 +998,7 @@ static int do_online_cores(char *cores, int state)
 			rc = -1;
 			continue;
 		}
-		if (core > cpus_in_system || core < 0) {
+		if (core >= cpus_in_system || core < 0) {
 			printf("Invalid core to %s: %d\n", state == 0 ? "offline" : "online", core);
 			rc = -1;
 			continue;
@@ -1235,20 +1047,12 @@ static int do_cores_on(char *state)
 		}
 	}
 
-	smt_state = get_smt_state();
-	if (smt_state == -1) {
-		printf("Bad or inconsistent SMT state: use ppc64_cpu --smt=on|off to set all\n"
-                       "cores to have the same number of online threads to continue.\n");
-		do_info();
-		return -1;
-	}
-
 	core_state = calloc(cpus_in_system, sizeof(int));
 	if (!core_state)
 		return -ENOMEM;
 
 	for (i = 0; i < cpus_in_system ; i++) {
-		core_state[i] = cpu_online(i * threads_per_cpu);
+		core_state[i] = (get_one_smt_state(i) > 0);
 		if (core_state[i])
 			cores_now_online++;
 	}
@@ -1259,6 +1063,14 @@ static int do_cores_on(char *state)
 		return 0;
 	}
 
+	smt_state = get_smt_state();
+	if (smt_state == -1) {
+		printf("Bad or inconsistent SMT state: use ppc64_cpu --smt=on|off to set all\n"
+                       "cores to have the same number of online threads to continue.\n");
+		do_info();
+		return -1;
+	}
+
 	if (!strcmp(state, "all")) {
 		number_to_have = cpus_in_system;
 	} else {
@@ -1363,7 +1175,7 @@ static void usage(void)
 {
 	printf(
 "Usage: ppc64_cpu [command] [options]\n"
-"ppc64_cpu --smt                     # Get current SMT state\n"
+"ppc64_cpu --smt [-n]                # Get current SMT state. [-n] shows numeric output\n"
 "ppc64_cpu --smt={on|off}            # Turn SMT on/off\n"
 "ppc64_cpu --smt=X                   # Set SMT state to X\n\n"
 "ppc64_cpu --cores-present           # Get the number of cores present\n"
@@ -1376,8 +1188,6 @@ static void usage(void)
 "ppc64_cpu --dscr=<val>              # Change DSCR system setting\n"
 "ppc64_cpu --dscr [-p <pid>]         # Get DSCR setting for process <pid>\n"
 "ppc64_cpu --dscr=<val> [-p <pid>]   # Change DSCR setting for process <pid>\n\n"
-"ppc64_cpu --smt-snooze-delay        # Get current smt-snooze-delay setting\n"
-"ppc64_cpu --smt-snooze-delay=<val>  # Change smt-snooze-delay setting\n\n"
 "ppc64_cpu --run-mode                # Get current diagnostics run mode\n"
 "ppc64_cpu --run-mode=<val>          # Set current diagnostics run mode\n\n"
 "ppc64_cpu --frequency [-t <time>]   # Determine cpu frequency for <time>\n"
@@ -1391,7 +1201,6 @@ static void usage(void)
 struct option longopts[] = {
 	{"smt",			optional_argument, NULL, 's'},
 	{"dscr",		optional_argument, NULL, 'd'},
-	{"smt-snooze-delay",	optional_argument, NULL, 'S'},
 	{"run-mode",		optional_argument, NULL, 'r'},
 	{"frequency",		no_argument,	   NULL, 'f'},
 	{"cores-present",	no_argument,	   NULL, 'C'},
@@ -1412,6 +1221,7 @@ int main(int argc, char *argv[])
 	char *equal_char;
 	int opt;
 	int sleep_time = 1; /* default to one second */
+	bool numeric = false;
 	pid_t pid = -1;
 
 	if (argc == 1) {
@@ -1419,7 +1229,7 @@ int main(int argc, char *argv[])
 		return 0;
 	}
 
-	rc = get_cpu_info();
+	rc = get_cpu_info(&threads_per_cpu, &cpus_in_system, &threads_in_system);
 	if (rc) {
 		printf("Could not determine system cpu/thread information.\n");
 		return rc;
@@ -1443,7 +1253,7 @@ int main(int argc, char *argv[])
 	/* Now parse out any additional options. */
 	optind = 2;
 	while (1) {
-		opt = getopt(argc, argv, "p:t:");
+		opt = getopt(argc, argv, "p:t:n");
 		if (opt == -1)
 			break;
 
@@ -1470,6 +1280,15 @@ int main(int argc, char *argv[])
 
 			sleep_time = atoi(optarg);
 			break;
+		case 'n':
+			if (strcmp(action, "smt")) {
+				fprintf(stderr, "The n option is only valid "
+					"with the --smt option\n");
+				usage();
+				exit(-1);
+			}
+			numeric = true;
+			break;
 		default:
 			fprintf(stderr, "%c is not a valid option\n", opt);
 			usage();
@@ -1478,11 +1297,9 @@ int main(int argc, char *argv[])
 	}
 
 	if (!strcmp(action, "smt"))
-		rc = do_smt(action_arg);
+		rc = do_smt(action_arg, numeric);
 	else if (!strcmp(action, "dscr"))
 		rc = do_dscr(action_arg, pid);
-	else if (!strcmp(action, "smt-snooze-delay"))
-		rc = do_smt_snooze_delay(action_arg);
 	else if (!strcmp(action, "run-mode"))
 		rc = do_run_mode(action_arg);
 	else if (!strcmp(action, "frequency"))
